#include<stdio.h>
#include<stdlib.h>
#include<stdbool.h>
#include"game.h"



// Définition de la structure du jeu

struct game_s{
  int nb_pieces;
  piece *pieces;
  int nb_moves;
  int width;
  int height;
};


// **Fonctions primitives de gestion du jeu**


/*Créé un nouveau jeu comprenant le nombre nb_pieces de pièces, listées dans le tableau de même nom.
Le nombre de mouvements du jeu est remis à zéro.*/
game new_game_hr (int nb_pieces, piece *pieces){
  game g = malloc(sizeof(*g));
  if(g == NULL){
    exit(EXIT_FAILURE);
  }
  g->nb_pieces = nb_pieces;
  g->pieces = malloc(sizeof(*(g->pieces))*nb_pieces);
  if(g->pieces == NULL){
    exit(EXIT_FAILURE);
  }
  g->pieces = pieces;
  g->nb_moves = 0;
  g->width=6;
  g->height=6;
  return g;
}

/*Supprime le jeu désigné en paramètre.
Libération de tous les champs de la structure.*/
void delete_game (game g){
  g->nb_pieces = 0;
  for(int i = 0 ; i < game_nb_pieces(g); i++){
    delete_piece(g->pieces[i]); 
  }
  free(g);
}

/*Récupère et retourne le nombre de pièces que comprend le jeu passé en paramètre.*/
int game_nb_pieces(cgame g){
  return g->nb_pieces;
}

/*Récupère et retourne, s'il y en a une, la pièce d'indice piece_num dans le tableau de pièces du jeu g passé en paramètre.*/
cpiece game_piece(cgame g, int piece_num){
  if(piece_num < 0 || piece_num > game_nb_pieces(g)-1){
    return NULL;
  }
  return g->pieces[piece_num];
}

/*Récupère et retourne le nombre de mouvements joués lors du jeu passé en paramètre.*/
int game_nb_moves(cgame g){
  return g->nb_moves;
}

/*Indique si le jeu est terminé (victoire).
Il y a victoire si la piece 0 a atteint la sortie: si ses coordonnées sont (4,3).*/
bool game_over_hr(cgame g){
  return (get_x(g->pieces[0]) == 4) && (get_y(g->pieces[0]) == 3);
}

/*Copie les champs du jeu constant source (src) vers le jeu destination (dst).
Nécessite que le jeu destination soit déjà créé.*/
void copy_game (cgame src, game dst){
  dst->nb_pieces = src->nb_pieces;
  dst->pieces = src->pieces;
  dst->nb_moves = src->nb_moves;
  dst->width = src->width;
  dst->height = src->height;
}

/*Indique si le déplacement de la pièce d'indice piece_num sur la grille de jeu est possible dans la direction d, pour la distance passée en paramètre.
La pièce ne doit pas sortir de la grille de jeu, de taille 6x6.
Retourne vrai si le déplacement est possible, faux sinon.
Le nombre de mouvement est augmenté du nombre de cases parcourues par la pièce.*/
bool play_move(game g, int piece_num, dir d, int distance){
  cpiece v = game_piece(g, piece_num);
  if(distance<=0 || piece_num<0 || piece_num>game_nb_pieces(g)-1){
    return false;
  }
  if (!can_move_y(v)){
    if(d == UP || d == DOWN){
      return false;
    }
    if(d == RIGHT && get_x(v) + get_width(v) + distance > game_width(g)){
      return false;
    }
    if(d == LEFT && get_x(v) - distance < 0){
      return false;
    }
  }
  else if(!can_move_x(v)){
    if(d == LEFT || d == RIGHT){
      return false;
    }
    if(d == UP && get_y(v) + get_height(v) + distance > game_height(g)){
      return false;
    }
    if(d == DOWN && get_y(v) - distance < 0){
      return false;
    }
  }
  else{
    if(d == UP && get_y(v) + get_height(v) + distance > game_height(g)){
      return false;
    }
    if(d == DOWN && get_y(v) - distance < 0){
      return false;
    }
    if(d == RIGHT && get_x(v) + get_width(v) + distance > game_width(g)){
      return false;
    }
    if(d == LEFT && get_x(v) - distance < 0){
      return false;
    }
  }
  piece piecetest = new_piece(0, 0, 1, 1, true, true);
  copy_piece(v, piecetest);
  for(int i=0 ; i<game_nb_pieces(g); i++){ //teste la collision avec une autre pièce sur la trajectoire.
    if(i != piece_num){
      for (int j=1 ; j<=distance ; j++){
	move_piece(piecetest, d, 1);
        if (intersect(piecetest, game_piece(g, i))){
	  return false;
        }
      }
    }
  }
  delete_piece(piecetest);
  move_piece(g->pieces[piece_num], d, distance); //effectue le déplacement si aucune collision détectée.
  g->nb_moves += distance;
  return true;
}

//////Spécial Ane Rouge//////

game new_game (int width, int height, int nb_pieces, piece *pieces){
  game g = malloc(sizeof(*g));
  if(g == NULL){
    exit(EXIT_FAILURE);
  }
  g->nb_pieces = nb_pieces;
  g->pieces = pieces;
  g->nb_moves = 0;
  g->width=width;
  g->height=height;
  return g;
}

int game_width(cgame g){
  return g->width;
}

int game_height(cgame g){
  return g->height;
}

int game_square_piece (game g, int x, int y){
  if((get_x(g->pieces[0])== x || get_x(g->pieces[0])== x-1) && (get_y(g->pieces[0])== y || get_y(g->pieces[0])== y-1)){
    return 0;
  }
  for(int i = 1 ; i <= g->nb_pieces-1 ; i++){
    if((get_x(g->pieces[i])<= x && get_x(g->pieces[i])+get_width(g->pieces[i])>x) && (get_y(g->pieces[i])<= y && get_y(g->pieces[i])+get_height(g->pieces[i])>y)){
      return i;
    }
  }
  return -1;
}

bool game_over(cgame g){
  return (get_x(g->pieces[0]) == 3) && (get_y(g->pieces[0]) == 0);
}

/*
Tests non exhaustifs de nos fonctions lors de leur création.

int main(void){
  piece p1 = new_piece_rh(0,3,true, true);
  piece p2 = new_piece_rh(3,0,false, false);
  piece t[2] = { p1 , p2 };
  game g = new_game(6,6,2 , t);
  if(g == NULL){
    printf("Plantage!!\n");
  }
  game h = new_game(6,6,0 , NULL);
  printf("......\n00....\n00....\n......\n...1..\n...1..\n");
  printf("longueur du jeu: %d\n",game_width(g));
  printf("hauteur du jeu: %d\n",game_height(g));
  for(int y = 5 ; y >= 0 ; y--){
    for(int x = 0 ; x < 6 ; x++){
      if(game_square_piece (g, x, y) == -1){
        printf(".");
      }
      else{
        printf("%d",game_square_piece (g, x, y));
      }
    }
    printf("\n");
  }
  copy_game(g,h);
  printf("g: %d %d\n", game_nb_pieces(g), game_nb_moves(g));
  printf("h: %d %d\n", h->nb_pieces, h->nb_moves);
  cpiece p3 = game_piece(g , 1);
  printf("p2 copie: %d %d\n", get_x(p3), get_y(p3));
  if(!play_move(g, 0, RIGHT, 4)){
    printf("Problème move\n");
  }
  p3 = game_piece(g , 0);
  printf("p1 copie: %d %d\n", get_x(p3), get_y(p3));
  if (game_over_hr(g)){
    printf("C'est vraiment cool!!\n");
  }
  else{
    printf("Souci !\n");
  }
  delete_game(g);
  delete_game(h);
  delete_piece(p1);
  delete_piece(p2);
  return EXIT_SUCCESS;
}*/
