#include<stdio.h>
#include<stdlib.h>
#include<stdbool.h>
#include"piece.h"


// Définition de la structure d'une pièce

struct piece_s{
  int abs;
  int ord;
  bool taille;
  bool sens;
  bool doublesens;
  int width;
  int height;
};



// **Fonctions primitives de gestion des pièces de jeu**


/* Créé une nouvelle pièce, de coordonnées (x,y) passées en paramètres.
La pièce occupe 2 cases si le booléen "small" est vrai, 3 cases sinon.
La pièce est horizontale si le booléen "horizontal" est vrai, verticale sinon.*/
piece new_piece_rh (int x, int y, bool small, bool horizontal){
  piece p = (piece)malloc(sizeof(*p));
  if(p == NULL){
    exit(EXIT_FAILURE);
  }
  p->abs = x;
  p->ord = y;
  p->taille = small;
  p->sens = horizontal;
  if(horizontal){
    p->height = 1;
    if(small){
      p->width = 2;
    }
    else{
      p->width = 3; 
    }
  }
  else{
    p->width = 1;
    if(small){
      p->height = 2;
    }
    else{
      p->height = 3; 
    }
  }
  return p;
}

/*Supprime la pièce désignée en paramètre.
  Libération de tous les champs de la structure: coordonnées (-1,-1) hors grille de jeu.*/
void delete_piece (piece p){
  free(p);
}

/*Récupère et retourne l'abscisse x de la pièce constante passée en paramètre.*/
int get_x(cpiece p){
  return p->abs;
}

/*Récupère et retourne l'ordonnée y de la pièce constante passée en paramètre.*/
int get_y(cpiece p){
  return p->ord;
}

/*Récupère et retourne la hauteur de la pièce constante passée en paramètre.
Si la pièce est horizontale, on considère que sa hauteur est de 1.
Si elle est verticale, on donnera sa taille selon la valeur du booléen "taille".*/
int get_height(cpiece p){
  return p->height;
}

/*Récupère et retourne la largeur de la pièce constante passée en paramètre.
Si la pièce est verticale, on considère que sa largeur est de 1.
Si elle est horizontale, on donnera sa taille selon la valeur du booléen "taille".*/
int get_width(cpiece p){
  return p->width;
}

/*Retourne vrai si la pièce est horizontale, faux sinon.*/
bool is_horizontal(cpiece p){
  return p->sens;
}

/*Copie les champs de la pièce constante source (src) vers la pièce destination (dst).
Nécessite que la pièce destination soit déjà créée.*/
void copy_piece (cpiece src, piece dst){
  dst->abs = src->abs;
  dst->ord = src->ord;
  dst->taille = src->taille;
  dst->sens = src->sens;
  dst->doublesens = src->doublesens;
  dst->width = src->width;
  dst->height = src->height;
}

/*Déplace la pièce p de la distance passée en paramètre, dans la direction d.
N'autorise que les déplacements dans le sens de la pièce (booléen "sens").*/
void move_piece (piece p, dir d, int distance){
  if(can_move_x(p) && can_move_y(p)){
    if(d == RIGHT){
      p->abs += distance;
    }
    else if(d == LEFT){
      p->abs -= distance;
    }
    else if(d == UP){
      p->ord += distance;
    }
    else if(d == DOWN){
      p->ord -= distance;
    }
  }
  else{
    if(is_horizontal(p)){
       if(d == RIGHT){
         p->abs += distance;
       }
       else if(d == LEFT){
         p->abs -= distance;
       }
       return;
    }
    if(!is_horizontal(p)){
      if(d == UP){
        p->ord += distance;
      }
      else if(d == DOWN){
        p->ord -= distance;
      }
      return;
    }
  }
}

/*Indique si deux pièces p1 et p2 passées en paramètres sont en collision.
Retourne vrai si elles le sont, faux sinon.
Selon le sens et la taille de chaque pièce, on compare une à une les coordonnées de toutes les cases qu'elles occupent.*/
bool intersect(cpiece p1, cpiece p2){
    bool intersect = false;
  for(int p1w = 0 ; p1w < get_width(p1) && !intersect; p1w ++){                // parcourt la largeur de p1
    for(int p2w = 0 ; p2w < get_width(p2) && !intersect; p2w ++){              // parcourt la largeur de p2
      for(int p1h = 0 ; p1h < get_height(p1) && !intersect; p1h ++){           // parcourt la hauteur de p1
	for(int p2h = 0 ; p2h < get_height(p2) && !intersect; p2h ++){         // parcourt la hauteur de p2
	  intersect = (p1->abs+p1w == p2->abs+p2w) && (p1->ord+p1h == p2->ord+p2h);
	}
      }
    }
  }
  return intersect;
}


////////////////////V2/////////////////////////


bool can_move_x(cpiece p){
  return p->doublesens || is_horizontal(p);
}

bool can_move_y(cpiece p){
  return p->doublesens || !is_horizontal(p);
}

piece new_piece (int x, int y, int width, int height, bool move_x, bool move_y){
  piece p = malloc(sizeof(*p));
  if(p == NULL){
    exit(EXIT_FAILURE);
  }
  p->abs = x;
  p->ord = y;
  p->height=height;
  p->width=width;
  if(move_x && move_y){
    p->doublesens = true;
    if(width == 1 && height == 1){
      p->taille = true; //taille true =  carré en V2
    }
    else{
      p->taille = false;
    }
  }
  else{
    p->doublesens = false;
  }
  if(width > height){
    p->sens = true;
  }
  else{
    p->sens = false; // /!\ un sens "false" signifie non-horizontal mais pas nécessairement vertical
  }
  return p;
}

/*
Tests non exhaustifs de nos fonctions lors de leur création.

int main(void){
  piece p = new_piece(0,1,1,1,true,true);
  printf("coordonnées de p : %d %d\n", p->abs,p->ord);
  piece q = new_piece(2,3,2,1,true,true);
  printf("......\n......\n..1...\n......\n0.....\n......\n\n");
  copy_piece (p, q);
  printf("......\n......\n......\n......\n1.....\n......\n");
  printf("coordonnées de q : %d %d\n", q->abs,q->ord);
  move_piece (p, RIGHT, 0);
  printf("......\n......\n......\n......\n10....\n.....\n");
  printf("p déplacée : %d %d\n", p->abs,p->ord);
  piece r = new_piece(1,0,2,1,true,true);
  printf("......\n......\n......\n......\n10....\n.22...\n");
  piece t[3] = {p,q,r};
  for(int i = 0; i<3; i++){
    if(can_move_x(t[i])){
      printf("x -> %d\n",i);
    }
    else{
      printf("=(\n");
    }
    if(can_move_y(t[i])){
      printf("y -> %d\n",i);
    }
    else{
      printf("=(\n");
    }
  }
  printf("version officiel!\n");
  bool ok = false;
  for(int y = 6; y>=0;y--){
    for(int x = 0; x<6; x++){
      for(int i = 0; i<3;i++){
        if((get_x(t[i])<=x && get_x(t[i])+get_width(t[i]) >x)  &&  (get_y(t[i])<=y && get_y(t[i])+get_height(t[i])>y)){
          printf("%d",i);
	  ok = true;
	}
      }
      if(!ok){
        printf(".");
      }
      else{
        ok=false;
      }
    }
    printf("\n");
  }
  if(intersect(p, r)){
    printf("intersection!\n");
  }
  else{
    printf("pas d'intersection!\n");
  }
  move_piece (p, DOWN, 1);
  move_piece (p, RIGHT, 1);
  if(intersect(p, r)){
    printf("intersection!\n");
  }
  else{
    printf("pas d'intersection!\n");
  }
  for(int y = 6; y>=0;y--){
    for(int x = 0; x<6; x++){
      for(int i = 0; i<3;i++){
        if((get_x(t[i])<=x && get_x(t[i])+get_width(t[i]) >x)  &&  (get_y(t[i])<=y && get_y(t[i])+get_height(t[i])>y)){
          printf("%d",i);
	  ok = true;
	}
      }
      if(!ok){
        printf(".");
      }
      else{
        ok=false;
      }
    }
    printf("\n");
  }
  delete_piece (p);
  printf("coordonnées de p : [%d; %d]\n", p->abs,p->ord);
  printf("abscisse de q : %d \n", get_x(q));
  printf("ordonnée de q : %d \n", get_y(q));
  printf("hauteur de q : %d \n", get_height(q));
  printf("longueur de q : %d \n", get_width(q));
  delete_piece(q);
  delete_piece(r);
  printf("c'est cool!\n");
  return 1;
}
*/
