#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include "game.h"
#include "piece.h"

#define NB_PIECES 4
piece pieces[NB_PIECES];

bool test_equality_int(int expected, int value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}

bool test_equality_bool(bool expected, bool value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}
void set_up() {
  pieces[0] = new_piece_rh(0, 3, true, true);
  pieces[1] = new_piece_rh(3, 0, true, false);
  pieces[2] = new_piece_rh(4, 1, true, true);
  pieces[3] = new_piece_rh(5, 3, false, false);
}

void tear_down() {
  for (int i = 0 ; i < 4; i++)
    delete_piece(pieces[i]);
}


bool test_new_game_hr(){
  bool result = true;
  game g = new_game_hr (4,pieces);
  set_up();
  for(int i = 0 ; i < 4 ; i ++){
    result = result && test_equality_int(4, game_nb_pieces(g), "new game");
    result = result && test_equality_int(0, game_nb_moves(g), "new game");
    result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "new game");
    result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "new game");
  }
  delete_game(g);
  tear_down();
  return result;
}

bool test_copy_game() {
  game g = new_game_hr (4,pieces);
  set_up();
  game h = new_game_hr(0, NULL);
  bool result = true;
  copy_game(g, h);
  for (int i = 0 ; i < 4; i++) {
    result = result && test_equality_int(game_nb_pieces(h), game_nb_pieces(g), "copy_game");
    result = result && test_equality_int(game_nb_moves(h), game_nb_moves(g), "copy_game");
    result = result && test_equality_int(get_x(game_piece(h, i)), get_x(game_piece(g, i)), "copy_game");
    result = result && test_equality_int(get_y(game_piece(h, i)), get_y(game_piece(g, i)), "copy_game");
  }
  delete_game(g);
  delete_game(h);
  tear_down();
  return result;
}
bool test_play_move(){
  game g = new_game_hr (4, pieces);
  set_up();
  bool result = true;
  piece q = new_piece_rh(0, 0, true, true);
  for(int i = 0; i < NB_PIECES-1; i++){			//parcours du tableau "pieces"
    for(int j = 0; j< 4; j++){                          //test des directions
      for(int dist = 0; dist < NB_PIECES; dist++){      //test des distances
	copy_piece(pieces[i], q);
	
	if(dist <= 0){
	  result = result && test_equality_bool(false, play_move(g, i, RIGHT, dist), "play_move distance nulle");
	}
//*****************
	else{
	  if(!j){    					//direction RIGHT
	    play_move(g, i, RIGHT, dist);
	    if(get_height(pieces[i]) > 1){		//verticale
	      result = result && test_equality_bool(false, play_move(g, i, RIGHT, dist), "play_move RIGHT1");
	      result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move RIGHT2");
	    }
	    else{						//horizontale
	      if(get_x(pieces[i]) + get_width(pieces[i]) + dist > 6){		//sortie du plateau
		result = result && test_equality_bool(false, play_move(g, i, RIGHT, dist), "play_move RIGHT3");
		result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move RIGHT4");
	      }
	      else{
		for(int k=0 ; k<game_nb_pieces(g); k++){
		  if(k != i){
		    for (int testdist=0 ; testdist<=dist ; testdist++){
		      move_piece(q, RIGHT, 1);
		      if (intersect(q, game_piece(g, i))){
			result = result && test_equality_bool(false, play_move(g, i, RIGHT, dist), "play_move RIGHT intersect");
			result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move RIGHT");
		      }
		    }
		  }
		}
		if (!result){
		  result = result && test_equality_bool(true, play_move(g, i, RIGHT, dist), "play_move RIGHT5");
		  result = result && test_equality_int(get_x(pieces[i])+dist, get_x(game_piece(g, i)), "play_move RIGHT6");
		  play_move(g, i, LEFT, dist);
		}
	      }
	    }
	    result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "play_move RIGHT7");
	  }
//**************
	  else if(j == 1){					//direction LEFT
	    play_move(g, i, LEFT, dist);	
	    if(get_height(pieces[i]) > 1){
	      result = result && test_equality_bool(false, play_move(g, i, LEFT, dist), "play_move LEFT");
	      result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move LEFT");
	    }
	    else{
	      if(get_x(pieces[i]) - dist < 0){
		result = result && test_equality_bool(false, play_move(g, i, LEFT, dist), "play_move LEFT");
		result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move LEFT");
	      }
	      else{
		for(int k=0 ; k<game_nb_pieces(g); k++){
		  if(k != i){
		    for (int testdist=0 ; testdist<=dist ; testdist++){
		      move_piece(q, LEFT, 1);
		      if (intersect(q, game_piece(g, i))){
			result = result && test_equality_bool(false, play_move(g, i, LEFT, dist), "play_move LEFT intersect");
			result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move LEFT");
		      }
		    }
		  }
		}
		if (!result){
		  result = result && test_equality_bool(true, play_move(g, i, LEFT, dist), "play_move LEFT");
		  result = result && test_equality_int(get_x(pieces[i])-dist, get_x(game_piece(g, i)), "play_move LEFT");
		  play_move(g, i, RIGHT, dist);
		}
	      }
	    }
	    result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "play_move LEFT");
	  }
//**********************
	  else if(j == 2){					//direction UP
	    play_move(g, i, UP, dist);	
	    if(get_width(pieces[i]) > 1){
	      result = result && test_equality_bool(false, play_move(g, i, UP, dist), "play_move UP");
	      result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "play_move UP");
	    }
	    else{
	      if(get_y(pieces[i]) + get_height(pieces[i]) + dist > 6){
		result = result && test_equality_bool(false, play_move(g, i, UP, dist), "play_move UP");
		result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "play_move UP");
	      }
	      else{
		for(int k=0 ; k<game_nb_pieces(g); k++){
		  if(k != i){
		    for (int testdist=0 ; testdist<=dist ; testdist++){
		      move_piece(q, LEFT, 1);
		      if (intersect(q, game_piece(g, i))){
			result = result && test_equality_bool(false, play_move(g, i, UP, dist), "play_move UP intersect");
			result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move UP");
		      }
		    }
		  }
		}
		if (!result){
		  result = result && test_equality_bool(true, play_move(g, i, UP, dist), "play_move UP");
		  result = result && test_equality_int(get_y(pieces[i])+dist, get_y(game_piece(g, i)), "play_move UP");
		  play_move(g, i, DOWN, dist);
		}
	      }
	    }
	    result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move UP ");
	  }

//*******************
	  else if(j == 3){       					//direction DOWN
	    play_move(g, i, DOWN, dist);
	    if(get_width(pieces[i]) > 1){
	      result = result && test_equality_bool(false, play_move(g, i, DOWN, dist), "play_move DOWN");
	      result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "play_move DOWN");
	    }
	    else{
	      if(get_y(pieces[i]) - dist < 0){
		result = result && test_equality_bool(false, play_move(g, i, DOWN, dist), "play_move DOWN");
		result = result && test_equality_int(get_y(pieces[i]), get_y(game_piece(g, i)), "play_move DOWN");
	      }
	      else{
		for(int k=0 ; k<game_nb_pieces(g); k++){
		  if(k != i){
		    for (int testdist=0 ; testdist<=dist ; testdist++){
		      move_piece(q, LEFT, 1);
		      if (intersect(q, game_piece(g, i))){
			result = result && test_equality_bool(false, play_move(g, i, DOWN, dist), "play_move DOWN intersect");
			result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move DOWN");
		      }
		    }
		  }
		}
		if (!result){
		  result = result && test_equality_bool(true, play_move(g, i, DOWN, dist), "play_move DOWN");
		  result = result && test_equality_int(get_y(pieces[i])-dist, get_y(game_piece(g, i)), "play_move DOWN");
		  play_move(g, i, UP, dist);
		}
	      }
	    }
	    result = result && test_equality_int(get_x(pieces[i]), get_x(game_piece(g, i)), "play_move DOWN");
	  }
	}
      }
    }
  }
  result = result && test_equality_int(4, game_nb_pieces(g), "play_move");
  delete_piece(q);
  delete_game(g);
  tear_down();
  return result;
}


int main(void){
  bool result = true;
  result = result && test_new_game_hr();
  result = result && test_copy_game();
  result = result && test_play_move();
  if(result){
    printf("Youpi!!\n");
    return 0;
  }
  else{
    printf("Oups!!\n");
    return 1;
  }
}

